@@ -1,13 +1,14 @@
+from box import Box
 from restfly.endpoint import APIEndpoint
 
 
 class AuditLogsAPI(APIEndpoint):
-    def status(self):
+    def status(self) -> Box:
         """
         Get the status of a request for an audit log report.
 
         Returns:
-            :obj:`dict`: Audit log report request status.
+            :obj:`Box`: Audit log report request status.
 
         Examples:
             >>> print(zia.audit_logs.status())
@@ -15,7 +16,7 @@ class AuditLogsAPI(APIEndpoint):
         """
         return self._get("auditlogEntryReport")
 
-    def create(self, start_time: str, end_time: str):
+    def create(self, start_time: str, end_time: str) -> int:
         """
         Creates an audit log report for the specified time period and saves it as a CSV file. The report
         includes audit information for every call made to the cloud service API during the specified time period.
@@ -28,7 +29,7 @@ class AuditLogsAPI(APIEndpoint):
                 The timestamp, in epoch, of the admin's last logout.
 
         Returns:
-            :obj:`str`: The status code for the operation.
+            :obj:`int`: The status code for the operation.
 
         Examples:
             >>> zia.audit_logs.create(start_time='1627221600000',
@@ -41,12 +42,12 @@ class AuditLogsAPI(APIEndpoint):
         }
         return self._post("auditlogEntryReport", json=payload, box=False).status_code
 
-    def cancel(self):
+    def cancel(self) -> int:
         """
         Cancels the request to create an audit log report.
 
         Returns:
-            :obj:`str`: The operation response code.
+            :obj:`int`: The operation response code.
 
         Examples:
             >>> zia.audit_logs.cancel()
@@ -54,7 +55,7 @@ class AuditLogsAPI(APIEndpoint):
         """
         return self._delete("auditlogEntryReport", box=False).status_code
 
-    def get_report(self):
+    def get_report(self) -> str:
         """
         Returns the most recently created audit log report.
 
@@ -68,4 +69,4 @@ class AuditLogsAPI(APIEndpoint):
             ...    fh.write(zia.audit_logs.get_report())
 
         """
-        return self._get("auditlogEntryReport/download", box=False).text
+        return self._get("auditlogEntryReport/download").text
